// Copyright 2007 (c) by Perforce Software, Inc.  All rights reserved.
//
// p4wChangeEditView:
//   The change edit view.


// -------------------------------------
// Includes
//
#include <stdlib.h>
#include <string.h>

#include <p4wp4.h>

#include "p4wView.h"
#include "p4wMenuPane.h"
#include "p4wChangeEditView.h"

#include "p4wChangePane.h"



//
// -------------------------------------
// Constructors and destructor.
//
p4wChangeEditView::p4wChangeEditView(p4wRequest & Request)
	: p4wView(Request)
{
	//
	// Set the short and the long versions of the title
	
	fShortTitle.Set( "Edit Change Spec" );
	fFullTitle << "Edit Change Spec " << fRequest.GetURL();
}

p4wChangeEditView::~p4wChangeEditView()
{
}



// -------------------------------------
// Control item render functions.
//

void p4wChangeEditView::RenderContent()
{
	//
	// Get the changelist number
	const StrPtr *changeNo = fRequest.GetCmdArg();
	if (!changeNo)
		changeNo = &fRequest.GetURL();
	if (!changeNo)
	{
		fRequest << "Invalid changelist number.";
		return;
	}

	//
	// Send the Change edit pane.
	p4wChangePane changePane(*this, fRequest, 1);
	const char *changePaneArgs[] = { "-o", changeNo->Text() };
	fRequest.p4("change", 2, changePaneArgs, &changePane);

	//
	// Wait for the pane to draw.
	fRequest.p4Wait();
}

void p4wChangeEditView::RenderMenu()
{
	//
	// Draw the menu bar
	p4wMenuPane menuPane(*this, fRequest, "editchange");
	menuPane.Render(NULL);
}
