//
// Copyright 2003 Perforce Software.  All rights reserved.
//
// This file is part of Perforce - the FAST SCM System.
//
// p4wView:
//   A view (single HTML page) in the p4w interface.

#ifndef p4wView_H
#define p4wView_H


#include "p4wRequest.h"
#include "p4wHtml.h"


class p4wView {

public:

	p4wView( p4wRequest & Request );
	virtual	~p4wView();


	//
	// Render this view.
	
	virtual	void Render();

	//
	// Control item render functions.
	
	virtual	void RenderRssHdrLink( p4wHtml *htm ) {};
	virtual	void RenderContent() {}
	virtual int RenderUpdateOK();
	virtual void RenderInfo();
	virtual void RenderMenu();
	virtual void StartContent();
	virtual void RenderCopyright();
	virtual void RenderError( char *error );
	
	void RenderTitle( int shortenTitle );
	void RenderFinalErrors();

	int IsFromFileBrowser() { return fIsFromFileBrowser; };
	
	int IsErrorPage() { return fIsErrorPage; };
	
	int  IsPreviewCmd() { return fPreview; };
	void SetIsPreviewCmd(int b) { fPreview = b; };
	
	int P4CharsProtected() { return fP4CharsProtected; }
		
	const char *GetOldPasswd() { return fOldPassword.Text(); }
	const char *GetNewPasswd() { return fNewPassword.Text(); }

protected:

	//
	// Render an error message that may contain links
	
	void RenderErrorLinks( char *error );

	//
	// Generate a Permalink
	
	void	GeneratePermalink( AllCommands ac );
	int		CantSimplifyPermalink(AllCommands ac, char *p, char *q);
	void	RemoveStateArg(char *arg, char *p, char *q);
	StrBuf	fPermalink;		// Permalink URL

	//
	// Our request class.
	
	p4wRequest &fRequest;
	
	//
	// Is this in the File browser mode
	
	int fIsFromFileBrowser;
	
	//
	// Does path contain pattern-matching chars that result 
	// in a directory (rather than a file)
	
        int HasDirPattern( const char *path );
        
        //
        // Short and long versions of the title
        
        StrBuf fShortTitle;
        StrBuf fFullTitle;

	int fIsErrorPage;
	
	//
	// Is this the result of a preview command?
	
	int fPreview;
	
	//
	// Did we protect characters special to p4 before
	// invoking the command?
	
	int fP4CharsProtected;
	
	//
	// Used for setting password
	
	StrBuf		fOldPassword;
	StrBuf		fNewPassword;
};

#endif // p4wView_H

